@@ -814,13 +814,12 @@ static int iw_get_encr(struct net_device *dev, struct iw_request_info *info,
 		WARNING("getting authentication mode failed (%08X)", res);
 		TRACEEXIT2(return -EOPNOTSUPP);
 	}
-
 	if (status == Ndis802_11AuthModeOpen)
 		wrqu->data.flags |= IW_ENCODE_OPEN;
-	else if (status == Ndis802_11AuthModeShared)
-		wrqu->data.flags |= IW_ENCODE_RESTRICTED;
 	else if (status == Ndis802_11AuthModeAutoSwitch)
 		wrqu->data.flags |= IW_ENCODE_RESTRICTED;
+	else // Ndis802_11AuthModeAutoSwitch, Ndis802_11AuthModeWPA etc.
+		wrqu->data.flags |= IW_ENCODE_RESTRICTED;
 
 	TRACEEXIT2(return 0);
 }
@@ -924,8 +923,8 @@ static int remove_key(struct wrap_ndis_device *wnd, int index,
 	TRACEEXIT2(return 0);
 }
 
-static int iw_set_encr(struct net_device *dev, struct iw_request_info *info,
-		       union iwreq_data *wrqu, char *extra)
+static int iw_set_wep(struct net_device *dev, struct iw_request_info *info,
+		      union iwreq_data *wrqu, char *extra)
 {
 	struct wrap_ndis_device *wnd = netdev_priv(dev);
 	NDIS_STATUS res;
@@ -1152,7 +1151,7 @@ static char *ndis_translate_scan(struct net_device *dev, char *event,
 	if (bssid->length > sizeof(*bssid)) {
 		unsigned char *iep = (unsigned char *)bssid_ex->ies +
 			sizeof(struct ndis_fixed_ies);
-		__unused unsigned char *end = iep + bssid_ex->ie_length;
+		no_warn_unused unsigned char *end = iep + bssid_ex->ie_length;
 
 		while (iep + 1 < end && iep + 2 + iep[1] <= end) {
 			unsigned char ielen = 2 + iep[1];
@@ -1523,27 +1522,62 @@ static int iw_get_range(struct net_device *dev, struct iw_request_info *info,
 	return 0;
 }
 
-#if WIRELESS_EXT > 17
-static int wpa_disassociate(struct net_device *dev,
-			    struct iw_request_info *info,
-			    union iwreq_data *wrqu, char *extra);
-static int wpa_deauthenticate(struct net_device *dev,
-			      struct iw_request_info *info,
-			      union iwreq_data *wrqu, char *extra);
+static int disassociate(struct wrap_ndis_device *wnd)
+{
+	u8 buf[NDIS_ESSID_MAX_SIZE];
+	int i;
 
+	miniport_set_info(wnd, OID_802_11_DISASSOCIATE, NULL, 0);
+	get_random_bytes(buf, sizeof(buf));
+	for (i = 0; i < sizeof(buf); i++)
+		buf[i] = 'a' + (buf[i] % 26);
+	set_essid(wnd, buf, sizeof(buf));
+	return 0;
+}
+
+
+static int deauthenticate(struct wrap_ndis_device *wnd)
+{
+	int ret;
+
+	TRACEENTER2("");
+	ret = disassociate(wnd);
+	set_priv_filter(wnd, Ndis802_11PrivFilterAcceptAll);
+	set_auth_mode(wnd, Ndis802_11AuthModeOpen);
+	set_encr_mode(wnd, Ndis802_11EncryptionDisabled);
+	TRACEEXIT2(return ret);
+}
+
+int set_priv_filter(struct wrap_ndis_device *wnd, int flags)
+{
+	NDIS_STATUS res;
+
+	TRACEENTER2("filter: %d", flags);
+	res = miniport_set_int(wnd, OID_802_11_PRIVACY_FILTER, flags);
+	if (res == NDIS_STATUS_FAILURE)
+		return -EOPNOTSUPP;
+	if (res == NDIS_STATUS_INVALID_DATA) {
+		WARNING("setting privacy filter to %d failed (%08X)",
+			flags, res);
+		TRACEEXIT2(return -EINVAL);
+	}
+	TRACEEXIT2(return 0);
+}
+
+#if WIRELESS_EXT > 17
 static int iw_set_mlme(struct net_device *dev, struct iw_request_info *info,
 		       union iwreq_data *wrqu, char *extra)
 {
+	struct wrap_ndis_device *wnd = netdev_priv(dev);
 	struct iw_mlme *mlme = (struct iw_mlme *)extra;
 
 	TRACEENTER2("");
 	switch (mlme->cmd) {
 	case IW_MLME_DEAUTH:
-		return wpa_deauthenticate(dev, info, wrqu, extra);
+		return deauthenticate(wnd);
 	case IW_MLME_DISASSOC:
-		DBGTRACE2("cmd=%d reason_code=%d",
-			  mlme->cmd, mlme->reason_code);
-		return wpa_disassociate(dev, info, wrqu, extra);
+		DBGTRACE2("cmd=%d reason_code=%d", mlme->cmd, mlme->reason_code);
+		return disassociate(wnd);
 	default:
 		return -EOPNOTSUPP;
 	}
@@ -1807,7 +1841,7 @@ static const iw_handler	ndis_handler[] = {
 	[SIOCSIWFRAG	- SIOCIWFIRST] = iw_set_frag_threshold,
 	[SIOCGIWAP	- SIOCIWFIRST] = iw_get_ap_address,
 	[SIOCSIWAP	- SIOCIWFIRST] = iw_set_ap_address,
-	[SIOCSIWENCODE	- SIOCIWFIRST] = iw_set_encr,
+	[SIOCSIWENCODE	- SIOCIWFIRST] = iw_set_wep,
 	[SIOCGIWENCODE	- SIOCIWFIRST] = iw_get_encr,
 	[SIOCSIWSCAN	- SIOCIWFIRST] = iw_set_scan,
 	[SIOCGIWSCAN	- SIOCIWFIRST] = iw_get_scan,
@@ -1976,7 +2010,8 @@ static int priv_set_auth_mode(struct net_device *dev,
 	TRACEEXIT2(return 0);
 }
 
-/* WPA support */
+#if WIRELESS_EXT <= 17
+/* WPA support through 'ndiswrapper' driver interface */
 
 static int wpa_init(struct net_device *dev, struct iw_request_info *info,
 		    union iwreq_data *wrqu, char *extra)
@@ -2143,14 +2178,7 @@ static int wpa_disassociate(struct net_device *dev,
 			    union iwreq_data *wrqu, char *extra)
 {
 	struct wrap_ndis_device *wnd = netdev_priv(dev);
-	u8 buf[NDIS_ESSID_MAX_SIZE];
-	int i;
-
-	miniport_set_info(wnd, OID_802_11_DISASSOCIATE, NULL, 0);
-	get_random_bytes(buf, sizeof(buf));
-	for (i = 0; i < sizeof(buf); i++)
-		buf[i] = 'a' + (buf[i] % 26);
-	set_essid(wnd, buf, sizeof(buf));
+	disassociate(wnd);
 	TRACEEXIT2(return 0);
 }
 
@@ -2163,10 +2191,6 @@ static int wpa_associate(struct net_device *dev, struct iw_request_info *info,
 	int infra_mode, auth_mode, encr_mode, priv_mode, size;
 
 	TRACEENTER2("");
-#if WIRELESS_EXT > 17
-	printk(KERN_INFO "%s: WPA support through 'ndiswrapper' interface "
-	       "is deprecated; use 'wext' interface\n", DRIVER_NAME);
-#endif
 	memset(&wpa_assoc_info, 0, sizeof(wpa_assoc_info));
 	wpa_assoc_info.mode = IEEE80211_MODE_INFRA;
 
@@ -2278,30 +2302,7 @@ static int wpa_deauthenticate(struct net_device *dev,
 			      union iwreq_data *wrqu, char *extra)
 {
 	struct wrap_ndis_device *wnd = netdev_priv(dev);
-	int ret;
-
-	TRACEENTER2("");
-	ret = wpa_disassociate(dev, info, wrqu, extra);
-	set_priv_filter(wnd, Ndis802_11PrivFilterAcceptAll);
-	set_auth_mode(wnd, Ndis802_11AuthModeOpen);
-	set_encr_mode(wnd, Ndis802_11EncryptionDisabled);
-	TRACEEXIT2(return ret);
-}
-
-int set_priv_filter(struct wrap_ndis_device *wnd, int flags)
-{
-	NDIS_STATUS res;
-
-	TRACEENTER2("filter: %d", flags);
-	res = miniport_set_int(wnd, OID_802_11_PRIVACY_FILTER, flags);
-	if (res == NDIS_STATUS_FAILURE)
-		return -EOPNOTSUPP;
-	if (res == NDIS_STATUS_INVALID_DATA) {
-		WARNING("setting privacy filter to %d failed (%08X)",
-			flags, res);
-		TRACEEXIT2(return -EINVAL);
-	}
-	TRACEEXIT2(return 0);
+	TRACEEXIT2(return deauthenticate(wnd));
 }
 
 static int wpa_set_priv_filter(struct net_device *dev,
@@ -2380,11 +2381,15 @@ static int wpa_get_capa(struct net_device *dev, struct iw_request_info *info,
 	TRACEEXIT2(return 0);
 }
 
+#endif // WIRELESS_EXT <= 17
+
 static const struct iw_priv_args priv_args[] = {
+#if WIRELESS_EXT <= 17
 	{WPA_DROP_UNENCRYPTED, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0,
 	 "set_priv_filter"},
 	{WPA_SET_AUTH_ALG, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0,
 	 "auth_alg"},
+#endif
 
 	{PRIV_RESET, 0, 0, "ndis_reset"},
 	{PRIV_POWER_PROFILE, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0,
@@ -2402,6 +2407,7 @@ static const struct iw_priv_args priv_args[] = {
 };
 
 static const iw_handler priv_handler[] = {
+#if WIRELESS_EXT <= 17
 	[WPA_SET_WPA 		- SIOCIWFIRSTPRIV] = wpa_set_wpa,
 	[WPA_SET_KEY 		- SIOCIWFIRSTPRIV] = wpa_set_key,
 	[WPA_ASSOCIATE 		- SIOCIWFIRSTPRIV] = wpa_associate,
@@ -2413,7 +2419,7 @@ static const iw_handler priv_handler[] = {
 	[WPA_INIT 		- SIOCIWFIRSTPRIV] = wpa_init,
 	[WPA_DEINIT 		- SIOCIWFIRSTPRIV] = wpa_deinit,
 	[WPA_GET_CAPA 		- SIOCIWFIRSTPRIV] = wpa_get_capa,
-
+#endif
 	[PRIV_RESET 		- SIOCIWFIRSTPRIV] = priv_reset,
 	[PRIV_POWER_PROFILE 	- SIOCIWFIRSTPRIV] = priv_power_profile,
 	[PRIV_NETWORK_TYPE 	- SIOCIWFIRSTPRIV] = priv_network_type,
