@@ -34,7 +34,7 @@ void misc_funcs_exit(void)
 	TRACEEXIT4(return);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_sprintf)
+noregparm INT WRAP_EXPORT(_win_sprintf)
 	(char *buf, const char *format, ...)
 {
 	va_list args;
@@ -46,13 +46,13 @@ NOREGPARM INT WRAP_EXPORT(_win_sprintf)
 	return res;
 }
 
-NOREGPARM INT WRAP_EXPORT(swprintf)
+noregparm INT WRAP_EXPORT(swprintf)
 	(wchar_t *buf, const wchar_t *format, ...)
 {
 	UNIMPL();
 	TRACEEXIT2(return 0);
 }
-NOREGPARM INT WRAP_EXPORT(_win_vsprintf)
+noregparm INT WRAP_EXPORT(_win_vsprintf)
 	(char *str, const char *format, va_list ap)
 {
 	INT i;
@@ -61,7 +61,7 @@ NOREGPARM INT WRAP_EXPORT(_win_vsprintf)
 	TRACEEXIT2(return i);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_snprintf)
+noregparm INT WRAP_EXPORT(_win_snprintf)
 	(char *buf, SIZE_T count, const char *format, ...)
 {
 	va_list args;
@@ -74,7 +74,7 @@ NOREGPARM INT WRAP_EXPORT(_win_snprintf)
 	return res;
 }
 
-NOREGPARM INT WRAP_EXPORT(_win__snprintf)
+noregparm INT WRAP_EXPORT(_win__snprintf)
 	(char *buf, SIZE_T count, const char *format, ...)
 {
 	va_list args;
@@ -87,7 +87,7 @@ NOREGPARM INT WRAP_EXPORT(_win__snprintf)
 	return res;
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_vsnprintf)
+noregparm INT WRAP_EXPORT(_win_vsnprintf)
 	(char *str, SIZE_T size, const char *format, va_list ap)
 {
 	INT i;
@@ -96,7 +96,7 @@ NOREGPARM INT WRAP_EXPORT(_win_vsnprintf)
 	TRACEEXIT2(return i);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win__vsnprintf)
+noregparm INT WRAP_EXPORT(_win__vsnprintf)
 	(char *str, SIZE_T size, const char *format, va_list ap)
 {
 	INT i;
@@ -105,43 +105,43 @@ NOREGPARM INT WRAP_EXPORT(_win__vsnprintf)
 	TRACEEXIT2(return i);
 }
 
-NOREGPARM char *WRAP_EXPORT(_win_strncpy)
+noregparm char *WRAP_EXPORT(_win_strncpy)
 	(char *dst, char *src, SIZE_T n)
 {
 	return strncpy(dst, src, n);
 }
 
-NOREGPARM SIZE_T WRAP_EXPORT(_win_strlen)
+noregparm SIZE_T WRAP_EXPORT(_win_strlen)
 	(const char *s)
 {
 	return strlen(s);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_strncmp)
+noregparm INT WRAP_EXPORT(_win_strncmp)
 	(const char *s1, const char *s2, SIZE_T n)
 {
 	return strncmp(s1, s2, n);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_strcmp)
+noregparm INT WRAP_EXPORT(_win_strcmp)
 	(const char *s1, const char *s2)
 {
 	return strcmp(s1, s2);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_stricmp)
+noregparm INT WRAP_EXPORT(_win_stricmp)
 	(const char *s1, const char *s2)
 {
 	return stricmp(s1, s2);
 }
 
-NOREGPARM char *WRAP_EXPORT(_win_strncat)
+noregparm char *WRAP_EXPORT(_win_strncat)
 	(char *dest, const char *src, SIZE_T n)
 {
 	return strncat(dest, src, n);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_wcscmp)
+noregparm INT WRAP_EXPORT(_win_wcscmp)
 	(const wchar_t *s1, const wchar_t *s2)
 {
 	while (*s1 != L'\0' && *s1 == *s2) {
@@ -151,7 +151,7 @@ NOREGPARM INT WRAP_EXPORT(_win_wcscmp)
 	return *s1 - *s2;
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_wcsicmp)
+noregparm INT WRAP_EXPORT(_win_wcsicmp)
 	(const wchar_t *s1, const wchar_t *s2)
 {
 	while (*s1 != L'\0' && tolower((char)*s1) == tolower((char)*s2)) {
@@ -161,7 +161,7 @@ NOREGPARM INT WRAP_EXPORT(_win_wcsicmp)
 	return *s1 - *s2;
 }
 
-NOREGPARM SIZE_T WRAP_EXPORT(_win_wcslen)
+noregparm SIZE_T WRAP_EXPORT(_win_wcslen)
 	(const wchar_t *s)
 {
 	SIZE_T i = 0;
@@ -170,7 +170,7 @@ NOREGPARM SIZE_T WRAP_EXPORT(_win_wcslen)
 	return i;
 }
 
-NOREGPARM wchar_t *WRAP_EXPORT(_win_wcsncpy)
+noregparm wchar_t *WRAP_EXPORT(_win_wcsncpy)
 	(wchar_t *dest, const wchar_t *src, SIZE_T n)
 {
 	SIZE_T i;
@@ -181,7 +181,7 @@ NOREGPARM wchar_t *WRAP_EXPORT(_win_wcsncpy)
 	return dest;
 }
 
-NOREGPARM wchar_t *WRAP_EXPORT(_win_wcscpy)
+noregparm wchar_t *WRAP_EXPORT(_win_wcscpy)
 	(wchar_t *dest, const wchar_t *src)
 {
 	SIZE_T i;
@@ -191,7 +191,7 @@ NOREGPARM wchar_t *WRAP_EXPORT(_win_wcscpy)
 	return dest;
 }
 
-NOREGPARM wchar_t *WRAP_EXPORT(_win_wcscat)
+noregparm wchar_t *WRAP_EXPORT(_win_wcscat)
 	(wchar_t *dest, const wchar_t *src)
 {
 	SIZE_T i;
@@ -205,129 +205,129 @@ NOREGPARM wchar_t *WRAP_EXPORT(_win_wcscat)
 	return dest;
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_towupper)
+noregparm INT WRAP_EXPORT(_win_towupper)
 	(wchar_t c)
 {
 	return toupper(c);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_towlower)
+noregparm INT WRAP_EXPORT(_win_towlower)
 	(wchar_t c)
 {
 	return tolower(c);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_tolower)
+noregparm INT WRAP_EXPORT(_win_tolower)
 	(INT c)
 {
 	return tolower(c);
 }
 
-NOREGPARM INT WRAP_EXPORT(_win_toupper)
+noregparm INT WRAP_EXPORT(_win_toupper)
 	(INT c)
 {
 	return toupper(c);
 }
 
-NOREGPARM void *WRAP_EXPORT(_win_strcpy)
+noregparm void *WRAP_EXPORT(_win_strcpy)
 	(void *to, const void *from)
 {
 	return strcpy(to, from);
 }
 
-NOREGPARM char *WRAP_EXPORT(_win_strstr)
+noregparm char *WRAP_EXPORT(_win_strstr)
 	(const char *s1, const char *s2)
 {
 	return strstr(s1, s2);
 }
 
-NOREGPARM char *WRAP_EXPORT(_win_strchr)
+noregparm char *WRAP_EXPORT(_win_strchr)
 	(const char *s, int c)
 {
 	return strchr(s, c);
 }
 
-NOREGPARM char *WRAP_EXPORT(_win_strrchr)
+noregparm char *WRAP_EXPORT(_win_strrchr)
 	(const char *s, int c)
 {
 	return strrchr(s, c);
 }
 
-NOREGPARM void *WRAP_EXPORT(_win_memmove)
+noregparm void *WRAP_EXPORT(_win_memmove)
 	(void *to, void *from, SIZE_T count)
 {
 	return memmove(to, from, count);
 }
 
-NOREGPARM void *WRAP_EXPORT(_win_memchr)
+noregparm void *WRAP_EXPORT(_win_memchr)
 	(const void *s, INT c, SIZE_T n)
 {
 	return memchr(s, c, n);
 }
 
 /* memcpy and memset are macros so we can't map them */
-NOREGPARM void *WRAP_EXPORT(_win_memcpy)
+noregparm void *WRAP_EXPORT(_win_memcpy)
 	(void *to, const void *from, SIZE_T n)
 {
 	return memcpy(to, from, n);
 }
 
-NOREGPARM void *WRAP_EXPORT(_win_memset)
+noregparm void *WRAP_EXPORT(_win_memset)
 	(void *s, char c, SIZE_T count)
 {
 	return memset(s, c, count);
 }
 
-NOREGPARM void WRAP_EXPORT(_win_srand)
+noregparm void WRAP_EXPORT(_win_srand)
 	(UINT seed)
 {
 	net_srandom(seed);
 }
 
-NOREGPARM int WRAP_EXPORT(_win_atoi)
+noregparm int WRAP_EXPORT(_win_atoi)
 	(const char *ptr)
 {
 	int i = simple_strtol(ptr, NULL, 10);
 	return i;
 }
 
-STDCALL int WRAP_EXPORT(_win_isprint)
+wstdcall int WRAP_EXPORT(_win_isprint)
 	(int c)
 {
 	return isprint(c);
 }
 
-STDCALL s64 WRAP_EXPORT(_alldiv)
+wstdcall s64 WRAP_EXPORT(_alldiv)
 	(s64 a, s64 b)
 {
 	return (a / b);
 }
 
-STDCALL u64 WRAP_EXPORT(_aulldiv)
+wstdcall u64 WRAP_EXPORT(_aulldiv)
 	(u64 a, u64 b)
 {
 	return (a / b);
 }
 
-STDCALL s64 WRAP_EXPORT(_allmul)
+wstdcall s64 WRAP_EXPORT(_allmul)
 	(s64 a, s64 b)
 {
 	return (a * b);
 }
 
-STDCALL u64 WRAP_EXPORT(_aullmul)
+wstdcall u64 WRAP_EXPORT(_aullmul)
 	(u64 a, u64 b)
 {
 	return (a * b);
 }
 
-STDCALL s64 WRAP_EXPORT(_allrem)
+wstdcall s64 WRAP_EXPORT(_allrem)
 	(s64 a, s64 b)
 {
 	return (a % b);
 }
 
-STDCALL u64 WRAP_EXPORT(_aullrem)
+wstdcall u64 WRAP_EXPORT(_aullrem)
 	(u64 a, u64 b)
 {
 	return (a % b);
@@ -357,7 +357,7 @@ __attribute__ ((regparm(3))) u64 WRAP_EXPORT(_aullshr)
 	return (a >> b);
 }
 
-STDCALL SIZE_T WRAP_EXPORT(RtlCompareMemory)
+wstdcall SIZE_T WRAP_EXPORT(RtlCompareMemory)
 	(const void *a, const void *b, SIZE_T len)
 {
 	size_t i;
@@ -382,25 +382,25 @@ STDCALL SIZE_T WRAP_EXPORT(RtlCompareMemory)
 	return i;
 }
 
-STDCALL void WRAP_EXPORT(RtlCopyMemory)
+wstdcall void WRAP_EXPORT(RtlCopyMemory)
 	(void *dst, const void *src, SIZE_T length)
 {
 	memcpy(dst, src, length);
 }
 
-STDCALL void WRAP_EXPORT(RtlZeroMemory)
+wstdcall void WRAP_EXPORT(RtlZeroMemory)
 	(void *dst, SIZE_T length)
 {
 	memset(dst, 0, length);
 }
 
-STDCALL void WRAP_EXPORT(RtlSecureZeroMemory)
+wstdcall void WRAP_EXPORT(RtlSecureZeroMemory)
 	(void *dst, SIZE_T length)
 {
 	memset(dst, 0, length);
 }
 
-STDCALL LONG WRAP_EXPORT(RtlCompareString)
+wstdcall LONG WRAP_EXPORT(RtlCompareString)
 	(const struct ansi_string *s1, const struct ansi_string *s2,
 	 BOOLEAN case_insensitive)
 {
@@ -423,7 +423,7 @@ STDCALL LONG WRAP_EXPORT(RtlCompareString)
 	TRACEEXIT2(return ret);
 }
 
-STDCALL LONG WRAP_EXPORT(RtlCompareUnicodeString)
+wstdcall LONG WRAP_EXPORT(RtlCompareUnicodeString)
 	(const struct unicode_string *s1, const struct unicode_string *s2,
 	 BOOLEAN case_insensitive)
 {
@@ -448,7 +448,7 @@ STDCALL LONG WRAP_EXPORT(RtlCompareUnicodeString)
 	TRACEEXIT2(return ret);
 }
 
-STDCALL BOOLEAN WRAP_EXPORT(RtlEqualString)
+wstdcall BOOLEAN WRAP_EXPORT(RtlEqualString)
 	(const struct ansi_string *s1, const struct ansi_string *s2,
 	 BOOLEAN case_insensitive)
 {
@@ -458,7 +458,7 @@ STDCALL BOOLEAN WRAP_EXPORT(RtlEqualString)
 	return !RtlCompareString(s1, s2, case_insensitive);
 }
 
-STDCALL BOOLEAN WRAP_EXPORT(RtlEqualUnicodeString)
+wstdcall BOOLEAN WRAP_EXPORT(RtlEqualUnicodeString)
 	(const struct unicode_string *s1, const struct unicode_string *s2,
 	 BOOLEAN case_insensitive)
 {
@@ -467,7 +467,7 @@ STDCALL BOOLEAN WRAP_EXPORT(RtlEqualUnicodeString)
 	return !RtlCompareUnicodeString(s1, s2, case_insensitive);
 }
 
-STDCALL void WRAP_EXPORT(RtlCopyUnicodeString)
+wstdcall void WRAP_EXPORT(RtlCopyUnicodeString)
 	(struct unicode_string *dst, struct unicode_string *src)
 {
 	TRACEENTER1("%p, %p", dst, src);
@@ -481,7 +481,7 @@ STDCALL void WRAP_EXPORT(RtlCopyUnicodeString)
 	TRACEEXIT1(return);
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(RtlAppendUnicodeToString)
+wstdcall NTSTATUS WRAP_EXPORT(RtlAppendUnicodeToString)
 	(struct unicode_string *dst, wchar_t *src)
 {
 	if (src) {
@@ -498,7 +498,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlAppendUnicodeToString)
 	return STATUS_SUCCESS;
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(RtlAppendUnicodeStringToString)
+wstdcall NTSTATUS WRAP_EXPORT(RtlAppendUnicodeStringToString)
 	(struct unicode_string *dst, struct unicode_string *src)
 {
 	if (dst->max_length < src->length + dst->length)
@@ -512,7 +512,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlAppendUnicodeStringToString)
 	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
-STDCALL ULONG WRAP_EXPORT(RtlxAnsiStringToUnicodeSize)
+wstdcall ULONG WRAP_EXPORT(RtlxAnsiStringToUnicodeSize)
 	(const struct ansi_string *string)
 {
 	int i;
@@ -522,7 +522,7 @@ STDCALL ULONG WRAP_EXPORT(RtlxAnsiStringToUnicodeSize)
 	return (i * sizeof(wchar_t));
 }
 
-STDCALL ULONG WRAP_EXPORT(RtlxUnicodeStringToAnsiSize)
+wstdcall ULONG WRAP_EXPORT(RtlxUnicodeStringToAnsiSize)
 	(const struct unicode_string *string)
 {
 	int i;
@@ -532,7 +532,7 @@ STDCALL ULONG WRAP_EXPORT(RtlxUnicodeStringToAnsiSize)
 	return i;
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(RtlAnsiStringToUnicodeString)
+wstdcall NTSTATUS WRAP_EXPORT(RtlAnsiStringToUnicodeString)
 	(struct unicode_string *dst, const struct ansi_string *src,
 	 BOOLEAN alloc)
 {
@@ -542,6 +542,11 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlAnsiStringToUnicodeString)
 	DBGTRACE2("%d, %d, %d, %d, %p", n, dst->max_length, src->length,
 		  src->max_length, src->buf);
 	if (alloc == TRUE) {
+		if (n == 0) {
+			dst->length = dst->max_length = 0;
+			dst->buf = NULL;
+			TRACEEXIT2(return STATUS_SUCCESS);
+		}
 		dst->max_length = n + sizeof(dst->buf[0]);
 		dst->buf = ExAllocatePoolWithTag(NonPagedPool,
 						 dst->max_length, 0);
@@ -563,7 +568,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlAnsiStringToUnicodeString)
 	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToAnsiString)
+wstdcall NTSTATUS WRAP_EXPORT(RtlUnicodeStringToAnsiString)
 	(struct ansi_string *dst, const struct unicode_string *src,
 	 BOOLEAN alloc)
 {
@@ -573,6 +578,11 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToAnsiString)
 	DBGTRACE2("%d, %d, %d, %d, %p", n, dst->max_length, src->length,
 		  src->max_length, src->buf);
 	if (alloc == TRUE) {
+		if (n == 0) {
+			dst->length = dst->max_length = 0;
+			dst->buf = NULL;
+			TRACEEXIT2(return STATUS_SUCCESS);
+		}
 		dst->max_length = n + sizeof(dst->buf[0]);
 		dst->buf = ExAllocatePoolWithTag(NonPagedPool,
 						 dst->max_length, 0);
@@ -593,7 +603,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToAnsiString)
 	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToInteger)
+wstdcall NTSTATUS WRAP_EXPORT(RtlUnicodeStringToInteger)
 	(struct unicode_string *ustring, ULONG base, ULONG *value)
 {
 	int i, negsign;
@@ -657,7 +667,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToInteger)
 	return STATUS_SUCCESS;
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(RtlIntegerToUnicodeString)
+wstdcall NTSTATUS WRAP_EXPORT(RtlIntegerToUnicodeString)
 	(ULONG value, ULONG base, struct unicode_string *ustring)
 {
 	wchar_t *buf = ustring->buf;
@@ -682,27 +692,27 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlIntegerToUnicodeString)
 	return STATUS_SUCCESS;
 }
 
-STDCALL LARGE_INTEGER WRAP_EXPORT(RtlConvertUlongToLargeInteger)
+wstdcall LARGE_INTEGER WRAP_EXPORT(RtlConvertUlongToLargeInteger)
 	(ULONG ul)
 {
 	LARGE_INTEGER li = ul;
 	return li;
 }
 
-_FASTCALL USHORT WRAP_EXPORT(RtlUShortByteSwap)
+wfastcall USHORT WRAP_EXPORT(RtlUShortByteSwap)
 	(USHORT src)
 {
 	return __swab16(src);
 }
 
-_FASTCALL ULONG WRAP_EXPORT(RtlUlongByteSwap)
+wfastcall ULONG WRAP_EXPORT(RtlUlongByteSwap)
 	(ULONG src)
 {
 	/* ULONG is 32 bits for both 32-bit and 64-bit architectures */
 	return __swab32(src);
 }
 
-STDCALL void WRAP_EXPORT(RtlInitUnicodeString)
+wstdcall void WRAP_EXPORT(RtlInitUnicodeString)
 	(struct unicode_string *dst, const wchar_t *src)
 {
 	TRACEENTER2("%p", dst);
@@ -722,7 +732,7 @@ STDCALL void WRAP_EXPORT(RtlInitUnicodeString)
 	TRACEEXIT1(return);
 }
 
-STDCALL void WRAP_EXPORT(RtlInitAnsiString)
+wstdcall void WRAP_EXPORT(RtlInitAnsiString)
 	(struct ansi_string *dst, const char *src)
 {
 	TRACEENTER2("%p", dst);
@@ -743,7 +753,7 @@ STDCALL void WRAP_EXPORT(RtlInitAnsiString)
 	TRACEEXIT2(return);
 }
 
-STDCALL void WRAP_EXPORT(RtlInitString)
+wstdcall void WRAP_EXPORT(RtlInitString)
 	(struct ansi_string *dst, const char *src)
 {
 	TRACEENTER2("%p", dst);
@@ -751,7 +761,7 @@ STDCALL void WRAP_EXPORT(RtlInitString)
 	TRACEEXIT2(return);
 }
 
-STDCALL void WRAP_EXPORT(RtlFreeUnicodeString)
+wstdcall void WRAP_EXPORT(RtlFreeUnicodeString)
 	(struct unicode_string *string)
 {
 	TRACEENTER2("%p", string);
@@ -764,7 +774,7 @@ STDCALL void WRAP_EXPORT(RtlFreeUnicodeString)
 	return;
 }
 
-STDCALL void WRAP_EXPORT(RtlFreeAnsiString)
+wstdcall void WRAP_EXPORT(RtlFreeAnsiString)
 	(struct ansi_string *string)
 {
 	TRACEENTER2("%p", string);
@@ -777,7 +787,7 @@ STDCALL void WRAP_EXPORT(RtlFreeAnsiString)
 	return;
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(RtlQueryRegistryValues)
+wstdcall NTSTATUS WRAP_EXPORT(RtlQueryRegistryValues)
 	(ULONG relative, wchar_t *path, struct rtl_query_registry_table *tbl,
 	 void *context, void *env)
 {
@@ -851,7 +861,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlQueryRegistryValues)
 	TRACEEXIT3(return ret);
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(RtlWriteRegistryValue)
+wstdcall NTSTATUS WRAP_EXPORT(RtlWriteRegistryValue)
 	(ULONG relative, wchar_t *path, wchar_t *name, ULONG type,
 	 void *data, ULONG length)
 {
@@ -876,13 +886,13 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlWriteRegistryValue)
 	TRACEEXIT5(return STATUS_SUCCESS);
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(RtlDeleteRegistryValue)
+wstdcall NTSTATUS WRAP_EXPORT(RtlDeleteRegistryValue)
 	(ULONG relative, wchar_t *path, wchar_t *name)
 {
 	return STATUS_SUCCESS;
 }
 
-STDCALL void WRAP_EXPORT(RtlAssert)
+wstdcall void WRAP_EXPORT(RtlAssert)
 	(char *failed_assertion, char *file_name, ULONG line_num,
 	 char *message)
 {
@@ -891,7 +901,7 @@ STDCALL void WRAP_EXPORT(RtlAssert)
 	return;
 }
 
-STDCALL int WRAP_EXPORT(rand)
+wstdcall int WRAP_EXPORT(rand)
 	(void)
 {
 	char buf[6];
