@@ -209,8 +209,8 @@ static NDIS_STATUS miniport_pnp_event(struct wrap_ndis_device *wnd,
 	enum ndis_power_profile power_profile;
 
 	TRACEENTER1("%p, %d", wnd, event);
-	/* RNDIS driver doesn't like to be notified if device is not
-	 * in initialized state */
+	/* RNDIS driver doesn't like to be notified if device is
+	 * already halted */
 	if (!test_bit(HW_INITIALIZED, &wnd->hw_status))
 		TRACEEXIT1(return NDIS_STATUS_SUCCESS);
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
@@ -285,12 +285,6 @@ static NDIS_STATUS miniport_init(struct wrap_ndis_device *wnd)
 	set_current_state(TASK_INTERRUPTIBLE);
 	schedule_timeout(1 * HZ);
 
-#if 1
-	status = miniport_set_power_state(wnd, NdisDeviceStateD0);
-	if (status)
-		DBGTRACE1("setting power state to device %s returns %08X",
-			  wnd->net_dev->name, status);
-#endif
 	set_bit(HW_INITIALIZED, &wnd->hw_status);
 	set_bit(HW_AVAILABLE, &wnd->hw_status);
 	status = miniport_query_info(wnd, OID_PNP_CAPABILITIES,
@@ -326,43 +320,64 @@ static NDIS_STATUS miniport_set_power_state(struct wrap_ndis_device *wnd,
 					    enum ndis_power_state state)
 {
 	NDIS_STATUS status;
-	struct miniport_char *miniport;
 
 	DBGTRACE1("state: %d", state);
 	if (state == NdisDeviceStateD0) {
 		status = STATUS_SUCCESS;
 		if (test_and_clear_bit(HW_HALTED, &wnd->hw_status)) {
 			DBGTRACE1("starting device");
-			/* TODO: should we use pnp_start_device instead? */
-			/* USB devices will be restarted by USB driver */
+			/* sis163u driver crashes when resuming from
+			 * s3; USB devices will be restarted by USB
+			 * driver anyway, so for now just ignore  */
 			if (wrap_is_usb_bus(wnd->wd->dev_bus_type))
 				status = NDIS_STATUS_SUCCESS;
-			else {
+			else
 				status = miniport_init(wnd);
-				if (status == NDIS_STATUS_SUCCESS) {
-					set_packet_filter(wnd,
-							  wnd->packet_filter);
-					set_multicast_list(wnd);
-				} else
-					WARNING("couldn't re-initialize device "
-						"%s", wnd->net_dev->name);
+		} else if (test_and_clear_bit(HW_SUSPENDED, &wnd->hw_status)) {
+			status = miniport_set_info(wnd, OID_PNP_SET_POWER,
+						   &state, (ULONG)sizeof(state));
+			if (status != NDIS_STATUS_SUCCESS)
+				WARNING("%s: setting power to state %d failed? "
+					"%08X", wnd->net_dev->name, state,
+					status);
+			status = NDIS_STATUS_SUCCESS;
+		}
+		if (status == NDIS_STATUS_SUCCESS) {
+			set_packet_filter(wnd, wnd->packet_filter);
+			set_multicast_list(wnd);
+			hangcheck_add(wnd);
+			add_stats_timer(wnd);
+			set_scan(wnd);
+			if (netif_running(wnd->net_dev)) {
+				netif_device_attach(wnd->net_dev);
+				netif_wake_queue(wnd->net_dev);
 			}
-			TRACEEXIT2(return status);
-		}
-		if (test_and_clear_bit(HW_SUSPENDED, &wnd->hw_status)) {
-			status = miniport_set_int(wnd, OID_PNP_SET_POWER,
-						  NdisDeviceStateD0);
-			DBGTRACE2("set_power returns %08X", status);
-			miniport = &wnd->wd->driver->ndis_driver->miniport;
-			if (status == NDIS_STATUS_SUCCESS)
-				miniport_pnp_event(wnd,
-				   NdisDevicePnPEventPowerProfileChanged);
-		}
+			netif_poll_enable(wnd->net_dev);
+		} else
+			WARNING("%s: couldn't set power to state %d; device not"
+				" resumed", wnd->net_dev->name, state);
 		TRACEEXIT1(return status);
 	} else {
 		status = NDIS_STATUS_NOT_SUPPORTED;
+		DBGTRACE2("irql: %d", current_irql());
+		netif_poll_disable(wnd->net_dev);
+		if (netif_running(wnd->net_dev)) {
+			netif_tx_disable(wnd->net_dev);
+			netif_device_detach(wnd->net_dev);
+		}
+		hangcheck_del(wnd);
+		del_stats_timer(wnd);
 		if (wnd->attributes & NDIS_ATTRIBUTE_NO_HALT_ON_SUSPEND) {
-			status = miniport_set_int(wnd, OID_PNP_SET_POWER, state);
+			if (wnd->ndis_wolopts) {
+				status = miniport_set_int(wnd,
+							  OID_PNP_ENABLE_WAKE_UP,
+							  wnd->ndis_wolopts);
+				if (status != NDIS_STATUS_SUCCESS)
+					WARNING("%s: couldn't enable WOL: %08x",
+						wnd->net_dev->name, status);
+			}
+			status = miniport_set_info(wnd, OID_PNP_SET_POWER,
+						   &state, (ULONG)sizeof(state));
 			if (status == NDIS_STATUS_SUCCESS)
 				set_bit(HW_SUSPENDED, &wnd->hw_status);
 			else
@@ -463,6 +478,9 @@ static struct ndis_packet *alloc_tx_packet(struct wrap_ndis_device *wnd,
 		oob_data->extension.info[TcpIpChecksumPacketInfo] =
 			&wnd->tx_csum_info;
 #endif
+	DBG_BLOCK(4) {
+		dump_bytes(__FUNCTION__, skb->data, skb->len);
+	}
 	return packet;
 }
 
@@ -763,8 +781,8 @@ static struct net_device_stats *ndis_get_stats(struct net_device *dev)
 static void ndis_set_multicast_list(struct net_device *dev)
 {
 	struct wrap_ndis_device *wnd = netdev_priv(dev);
-	set_bit(SET_MULTICAST_LIST, &wnd->wrap_ndis_work);
-	schedule_wrap_work(&wnd->wrap_ndis_worker);
+	set_bit(SET_MULTICAST_LIST, &wnd->wrap_ndis_pending_work);
+	schedule_wrap_work(&wnd->wrap_ndis_work);
 }
 
 /* this function is called fom BH context */
@@ -798,9 +816,42 @@ static u32 ndis_get_link(struct net_device *dev)
 	return wnd->link_status;
 }
 
+static void ndis_get_wol(struct net_device *dev, struct ethtool_wolinfo *wol)
+{
+	struct wrap_ndis_device *wnd = netdev_priv(dev);
+	if (wnd->ndis_wolopts & NDIS_PNP_WAKE_UP_MAGIC_PACKET)
+		wol->wolopts |= WAKE_MAGIC;
+	/* no other options supported */
+	return;
+}
+
+static int ndis_set_wol(struct net_device *dev, struct ethtool_wolinfo *wol)
+{
+	struct wrap_ndis_device *wnd = netdev_priv(dev);
+	struct ndis_pnp_capabilities pnp_capa;
+	NDIS_STATUS status;
+
+	if (!(wol->wolopts & WAKE_MAGIC))
+		return -EINVAL;
+	if (!(wnd->attributes & NDIS_ATTRIBUTE_NO_HALT_ON_SUSPEND))
+		return -EOPNOTSUPP;
+	status = miniport_query_info(wnd, OID_PNP_CAPABILITIES,
+				     &pnp_capa, sizeof(pnp_capa));
+	if (status != NDIS_STATUS_SUCCESS)
+		return -EOPNOTSUPP;
+	/* we always suspend to D3 */
+	if (pnp_capa.wakeup_capa.min_magic_packet_wakeup != NdisDeviceStateD3)
+		return -EOPNOTSUPP;
+	/* no other options supported */
+	wnd->ndis_wolopts = NDIS_PNP_WAKE_UP_MAGIC_PACKET;
+	return 0;
+}
+
 static struct ethtool_ops ndis_ethtool_ops = {
 	.get_drvinfo		= ndis_get_drvinfo,
 	.get_link		= ndis_get_link,
+	.get_wol		= ndis_get_wol,
+	.set_wol		= ndis_set_wol,
 };
 #endif
 
@@ -988,7 +1039,6 @@ static void link_status_handler(struct wrap_ndis_device *wnd)
 
 	memset(&wrqu, 0, sizeof(wrqu));
 	wrqu.data.length = p - wpa_assoc_info;
-	DBGTRACE2("adding %d bytes", wrqu.data.length);
 	wireless_send_event(wnd->net_dev, IWEVCUSTOM, &wrqu, wpa_assoc_info);
 
 	kfree(wpa_assoc_info);
@@ -998,7 +1048,7 @@ static void link_status_handler(struct wrap_ndis_device *wnd)
 	get_ap_address(wnd, (char *)&wrqu.ap_addr.sa_data);
 	wrqu.ap_addr.sa_family = ARPHRD_ETHER;
 	wireless_send_event(wnd->net_dev, SIOCGIWAP, &wrqu, NULL);
-	DBGTRACE2("associate to " MACSTR, MAC2STR(wrqu.ap_addr.sa_data));
+	DBGTRACE2("associatedd to " MACSTR, MAC2STR(wrqu.ap_addr.sa_data));
 	TRACEEXIT2(return);
 }
 
@@ -1006,8 +1056,8 @@ static void stats_timer_proc(unsigned long data)
 {
 	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 
-	set_bit(COLLECT_STATS, &wnd->wrap_ndis_work);
-	schedule_wrap_work(&wnd->wrap_ndis_worker);
+	set_bit(COLLECT_STATS, &wnd->wrap_ndis_pending_work);
+	schedule_wrap_work(&wnd->wrap_ndis_work);
 	wnd->stats_timer.expires += 10 * HZ;
 	add_timer(&wnd->stats_timer);
 }
@@ -1033,8 +1083,8 @@ static void hangcheck_proc(unsigned long data)
 	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 
 	TRACEENTER3("");
-	set_bit(HANGCHECK, &wnd->wrap_ndis_work);
-	schedule_wrap_work(&wnd->wrap_ndis_worker);
+	set_bit(HANGCHECK, &wnd->wrap_ndis_pending_work);
+	schedule_wrap_work(&wnd->wrap_ndis_work);
 
 	wnd->hangcheck_timer.expires += wnd->hangcheck_interval;
 	add_timer(&wnd->hangcheck_timer);
@@ -1063,26 +1113,27 @@ void hangcheck_del(struct wrap_ndis_device *wnd)
 }
 
 /* worker procedure to take care of setting/checking various states */
-static void wrap_ndis_worker_proc(void *param)
+static void wrap_ndis_worker(void *param)
 {
 	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)param;
 
-	DBGTRACE2("%lu", wnd->wrap_ndis_work);
+	DBGTRACE2("%lu", wnd->wrap_ndis_pending_work);
 
-	if (test_bit(SHUTDOWN, &wnd->wrap_ndis_work) ||
+	if (test_bit(SHUTDOWN, &wnd->wrap_ndis_pending_work) ||
 	    !test_bit(HW_INITIALIZED, &wnd->hw_status))
 		TRACEEXIT3(return);
 
-	if (test_and_clear_bit(SET_MULTICAST_LIST, &wnd->wrap_ndis_work))
+	if (test_and_clear_bit(SET_MULTICAST_LIST, &wnd->wrap_ndis_pending_work))
 		set_multicast_list(wnd);
 
-	if (test_and_clear_bit(COLLECT_STATS, &wnd->wrap_ndis_work))
+	if (test_and_clear_bit(COLLECT_STATS, &wnd->wrap_ndis_pending_work))
 		update_wireless_stats(wnd);
 
-	if (test_and_clear_bit(LINK_STATUS_CHANGED, &wnd->wrap_ndis_work))
+	if (test_and_clear_bit(LINK_STATUS_CHANGED,
+			       &wnd->wrap_ndis_pending_work))
 		link_status_handler(wnd);
 
-	if (test_and_clear_bit(HANGCHECK, &wnd->wrap_ndis_work)) {
+	if (test_and_clear_bit(HANGCHECK, &wnd->wrap_ndis_pending_work)) {
 		NDIS_STATUS res;
 		struct miniport_char *miniport;
 		KIRQL irql;
@@ -1235,8 +1286,8 @@ void get_encryption_capa(struct wrap_ndis_device *wnd)
 
 /* called as Windows function, so call WIN2LIN2 before accessing
  * arguments */
-STDCALL NTSTATUS NdisDispatchDeviceControl(struct device_object *fdo,
-					   struct irp *irp)
+wstdcall NTSTATUS NdisDispatchDeviceControl(struct device_object *fdo,
+					    struct irp *irp)
 {
 	struct wrap_ndis_device *wnd;
 
@@ -1251,7 +1302,7 @@ STDCALL NTSTATUS NdisDispatchDeviceControl(struct device_object *fdo,
 
 /* called as Windows function, so call WIN2LIN2 before accessing
  * arguments */
-STDCALL NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp)
+wstdcall NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp)
 {
 	struct io_stack_location *irp_sl;
 	struct wrap_ndis_device *wnd;
@@ -1265,10 +1316,13 @@ STDCALL NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp)
 	wnd = fdo->reserved;
 	IOTRACE("fdo: %p, fn: %d:%d, wnd: %p", fdo, irp_sl->major_fn,
 		irp_sl->minor_fn, wnd);
-	if (irp_sl->params.power.state.device_state == PowerDeviceD0)
-		state = NdisDeviceStateD0;
-	else
+	if ((irp_sl->params.power.type == SystemPowerState &&
+	     irp_sl->params.power.state.system_state > PowerSystemWorking) ||
+	    (irp_sl->params.power.type == DevicePowerState &&
+	     irp_sl->params.power.state.device_state > PowerDeviceD0))
 		state = NdisDeviceStateD3;
+	else
+		state = NdisDeviceStateD0;
 	switch (irp_sl->minor_fn) {
 	case IRP_MN_SET_POWER:
 		if (state == NdisDeviceStateD0) {
@@ -1279,14 +1333,6 @@ STDCALL NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp)
 			if (ndis_status != NDIS_STATUS_SUCCESS)
 				WARNING("setting power to %d failed: %08X",
 					state, ndis_status);
-			hangcheck_add(wnd);
-			add_stats_timer(wnd);
-			set_scan(wnd);
-			if (netif_running(wnd->net_dev)) {
-				netif_device_attach(wnd->net_dev);
-				netif_wake_queue(wnd->net_dev);
-			}
-			netif_poll_enable(wnd->net_dev);
 			DBGTRACE2("%s: device resumed", wnd->net_dev->name);
 			irp->io_status.status = status = STATUS_SUCCESS;
 			IoCompleteRequest(irp, IO_NO_INCREMENT);
@@ -1295,14 +1341,6 @@ STDCALL NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp)
 			if (test_bit(HW_SUSPENDED, &wnd->hw_status) ||
 			    test_bit(HW_HALTED, &wnd->hw_status))
 				return STATUS_FAILURE;
-			DBGTRACE2("irql: %d", current_irql());
-			netif_poll_disable(wnd->net_dev);
-			if (netif_running(wnd->net_dev)) {
-				netif_tx_disable(wnd->net_dev);
-				netif_device_detach(wnd->net_dev);
-			}
-			hangcheck_del(wnd);
-			del_stats_timer(wnd);
 			ndis_status = miniport_set_power_state(wnd, state);
 			/* TODO: handle error case */
 			if (ndis_status != NDIS_STATUS_SUCCESS)
@@ -1312,14 +1350,19 @@ STDCALL NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp)
 		}
 		break;
 	case IRP_MN_QUERY_POWER:
-		ndis_status = miniport_query_int(wnd, OID_PNP_QUERY_POWER,
-						 &state);
-		DBGTRACE2("query_power to state %d returns %08X",
-			  state, ndis_status);
-		if (ndis_status == NDIS_STATUS_SUCCESS)
-			status = LIN2WIN2(IoPassIrpDown, wnd->nmb->pdo, irp);
-		else
-			status = STATUS_FAILURE;
+		if (!(wnd->attributes & NDIS_ATTRIBUTE_NO_HALT_ON_SUSPEND)) {
+			ndis_status = miniport_query_info(wnd,
+							  OID_PNP_QUERY_POWER,
+							  &state, sizeof(state));
+			DBGTRACE2("query_power to state %d returns %08X",
+				  state, ndis_status);
+			if (ndis_status == NDIS_STATUS_SUCCESS)
+				irp->io_status.status = STATUS_SUCCESS;
+			else
+				irp->io_status.status = STATUS_FAILURE;
+		} else
+			irp->io_status.status = STATUS_FAILURE;
+		status = LIN2WIN2(IoPassIrpDown, wnd->nmb->pdo, irp);
 		break;
 	case IRP_MN_WAIT_WAKE:
 	case IRP_MN_POWER_SEQUENCE:
@@ -1335,7 +1378,7 @@ STDCALL NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp)
 
 /* called as Windows function, so call WIN2LIN2 before accessing
  * arguments */
-STDCALL NTSTATUS NdisDispatchPnp(struct device_object *fdo, struct irp *irp)
+wstdcall NTSTATUS NdisDispatchPnp(struct device_object *fdo, struct irp *irp)
 {
 	struct io_stack_location *irp_sl;
 	struct wrap_ndis_device *wnd;
@@ -1371,6 +1414,9 @@ STDCALL NTSTATUS NdisDispatchPnp(struct device_object *fdo, struct irp *irp)
 		break;
 	case IRP_MN_REMOVE_DEVICE:
 		DBGTRACE1("%s", wnd->net_dev->name);
+		if (wnd->wd->surprise_removed == TRUE)
+			miniport_pnp_event(wnd,
+					   NdisDevicePnPEventSurpriseRemoved);
 		if (ndis_remove_device(wnd)) {
 			status = STATUS_FAILURE;
 			break;
@@ -1391,13 +1437,13 @@ STDCALL NTSTATUS NdisDispatchPnp(struct device_object *fdo, struct irp *irp)
 static int set_task_offload(struct wrap_ndis_device *wnd, void *buf,
 			    const int buf_size)
 {
+#if 0
 	struct ndis_task_offload_header *task_offload_header;
 	struct ndis_task_offload *task_offload;
 	struct ndis_task_tcp_ip_checksum *task_tcp_ip_csum = NULL;
 	struct ndis_task_tcp_ip_checksum csum;
 	NDIS_STATUS status;
 
-	TRACEEXIT1(return 0);
 	memset(buf, 0, buf_size);
 	task_offload_header = buf;
 	task_offload_header->version = NDIS_TASK_OFFLOAD_VERSION;
@@ -1465,7 +1511,8 @@ static int set_task_offload(struct wrap_ndis_device *wnd, void *buf,
 		}
 	}
 	wnd->rx_csum = csum.v4_rx;
-	return 0;
+#endif
+	TRACEEXIT1(return 0);
 }
 
 static void get_supported_oids(struct wrap_ndis_device *wnd)
@@ -1691,14 +1738,14 @@ static int ndis_remove_device(struct wrap_ndis_device *wnd)
 {
 	int tx_pending;
 
-	set_bit(SHUTDOWN, &wnd->wrap_ndis_work);
+	set_bit(SHUTDOWN, &wnd->wrap_ndis_pending_work);
 	wnd->tx_ok = 0;
 	ndis_close(wnd->net_dev);
 	netif_carrier_off(wnd->net_dev);
+	cancel_delayed_work(&wnd->wrap_ndis_work);
 	/* In 2.4 kernels, this function is called in atomic context,
-	 * so we can't/don't need to wait on mutex. */
+	 * so we can't (don't need to?) wait on mutex. */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-	cancel_delayed_work(&wnd->wrap_ndis_worker);
 	down_interruptible(&wnd->tx_ring_mutex);
 #endif
 	tx_pending = wnd->tx_ring_end - wnd->tx_ring_start;
@@ -1719,8 +1766,6 @@ static int ndis_remove_device(struct wrap_ndis_device *wnd)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	up(&wnd->tx_ring_mutex);
 #endif
-	if (wnd->wd->surprise_removed == TRUE)
-		miniport_pnp_event(wnd, NdisDevicePnPEventSurpriseRemoved);
 	wrap_procfs_remove_ndis_device(wnd);
 	miniport_halt(wnd);
 	ndis_exit_device(wnd);
@@ -1742,7 +1787,7 @@ static int ndis_remove_device(struct wrap_ndis_device *wnd)
 	TRACEEXIT2(return 0);
 }
 
-static STDCALL NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
+static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 				      struct device_object *pdo)
 {
 	struct device_object *fdo;
@@ -1824,11 +1869,11 @@ static STDCALL NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	init_timer(&wnd->hangcheck_timer);
 	wnd->scan_timestamp = 0;
 	init_timer(&wnd->stats_timer);
-	wnd->wrap_ndis_work = 0;
+	wnd->wrap_ndis_pending_work = 0;
 	memset(&wnd->essid, 0, sizeof(wnd->essid));
 	memset(&wnd->encr_info, 0, sizeof(wnd->encr_info));
 	wnd->infrastructure_mode = Ndis802_11Infrastructure;
-	INIT_WORK(&wnd->wrap_ndis_worker, wrap_ndis_worker_proc, wnd);
+	INIT_WORK(&wnd->wrap_ndis_work, wrap_ndis_worker, wnd);
 	wnd->hw_status = 0;
 	set_bit(HW_AVAILABLE, &wnd->hw_status);
 	if (wd->driver->ndis_driver)
